#! /bin/bash
# NetworkBench agent installation script.

#1、指定当前安装的版本号
: ${nbversion:="2.9.1"}

#函数,解析k=v,返回v
parse_val() {
  local oIFS=$IFS
  local ID=0
  local result=
  IFS='='
  for v in $1; do
    [ $ID = "1" ] && {
      result=$v
      break
    }
    ID=`expr $ID + 1`
  done
  IFS=$oIFS
  echo "$result"
}
#2、解析prefix参数
prefix=/usr
parse_param() {
  while [ -n "$1" ]; do
    case $1 in
      prefix=* | -prefix=* | --prefix=*) prefix=`parse_val $1`;;
      php_path=*) php_path=`parse_val $1`;;
      *);;
    esac
    shift
  done
}
parse_param $@



#换行符
nl='
'

if echo ' \c' | grep 'c' > /dev/null 2>&1; then
    en='-n'
    ec=
else
    en=
    ec='\c'
fi

#命令行参数解析,安装 or 卸载
imode=
[ -n "$1" ] && [ "$1" = "install" -o "$1" = "uninstall" ] && imode=$1

# 获取操作系统版本
ostype=
if [ -f /etc/redhat-release -o -f /etc/redhat_version ]; then
    ostype=rhel
elif [ -d /etc/dpkg ]; then
    ostype=debian
elif [ -f /etc/release ]; then
    if grep Solaris /etc/release > /dev/null 2>&1; then
  ostype=solaris
    fi
fi
if [ -z "${ostype}" ]; then
    tus=`uname -s 2> /dev/null`
    case "${tus}" in
  [Dd][Aa][Rr][Ww][Ii][Nn])     ostype=darwin ;;
  [Ff][Rr][Ee][Ee][Bb][Ss][Dd]) ostype=freebsd ;;
  [Ss][Uu][Nn][Oo][Ss])         ostype=solaris ;;
  [Ss][Mm][Aa][Rr][Tt][Oo][Ss]) ostype=solaris ;;
    esac
fi

: ${ostype:=generic}
: ${PATH_SEPARATOR:=':'}

#定位安装位置
ilibdir="${prefix}/lib/networkbench"

# 检测操作系统位数(x86 or x64) 和发行版名字
arch=`(uname -m) 2> /dev/null` || arch="unknown"
os=`(uname -s) 2> /dev/null` || os="unknown"
case "${arch}" in
    i[3456789]86)         arch=x86 ;;
    *64* | *amd*)         arch=x64 ;;
    i86pc)
    if [ "${ostype}" = "solaris" ]; then
      solarch=`/usr/bin/isainfo -k 2> /dev/null`
      case "$solarch" in
          *64*)           arch=x64 ;;
          *)              arch=x86 ;;
      esac
        else
      arch=x86
    fi
    ;;
    unknown)              arch=x86 ;;
esac

case "${os}" in
    [dD]arwin* ) arch="universal" ;;
    *) ;;
esac

# 检测安装文件的完整性
dmissing=
fmissing=
check_dir() {
    [ -d $1 ] || dmissing="${1}${nl}${dmissing}"
}
check_file() {
    [ -f $1 ] || fmissing="${1}${nl}${fmissing}"
}


daemon="${prefix}/bin/networkbench"

check_file "${daemon}"
# 检测文件夹是否存在
for d in "${ilibdir}/scripts" "${ilibdir}/agent"; do
  check_dir "${d}"
done

[ "${arch}" = "x64" ] && check_dir "${ilibdir}/agent/x64"
[ "${arch}" = "x86" ] && check_dir "${ilibdir}/agent/x86"
if [ -n "${dmissing}" ]; then
    echo "ERROR: the following directories could not be found:" >&2
    echo "${dmissing}" | sed -e 's/^/   /' >&2
fi

# 检测文件是否存在
check_file "${ilibdir}/scripts/networkbench.ini.template"
for pmv in "20090626" "20100525" "20121212" "20131226" "20151012" "20160303"; do
    check_file "${ilibdir}/agent/${arch}/networkbench-${pmv}.so"
    check_file "${ilibdir}/agent/${arch}/networkbench-${pmv}-zts.so"    
done
if [ -n "${fmissing}" ]; then
    echo "ERROR: the following files could not be found:" >&2
    echo "${fmissing}" | sed -e 's/^/   /' >&1
fi
if [ -n "${dmissing}" -o -n "${fmissing}" ]; then
    cat <<EOF
安装失败：安装文件内缺少文件，请重新下载安装包

Fail: The files or directories listed above could not be found. 
Please re-download Networkbench agent and try again.

EOF
    exit 1
fi
chmod u+s ${daemon}

# 开始安装: 首先指定退出时的动作: 打包日志
done_msg=
success=
cleanup() {
    (cd /tmp
  tar cf "${logtar}" nbinstall-${nbitemp}.log nbinstall-${nbitemp}.sysinfo > /dev/null 2>&1)
    rm -f ${nbilog} ${nbsysinfo} > /dev/null 2>&1
    [ -n success -a -n done_msg ] && echo -e "\033[31m ${done_msg}" ; tput sgr0
}
#退出信号处理过程:cleanup
trap cleanup EXIT
#生成执行日志文件名
nbitemp=`date +%Y%m%d-%H%M%S 2> /dev/null`
nbitemp="${nbitemp}-$$"
nbilog=/tmp/nbinstall-${nbitemp}.log
nbsysinfo=/tmp/nbinstall-${nbitemp}.sysinfo
logtar=/tmp/nbinstall-${nbitemp}.tar
#取开始时间
now=`date 2>/dev/null`
#初始化日志文件,清理前次遗留文件
rm -f $nbilog $nbsysinfo $logtar > /dev/null 2>&1
echo "Networkbench Agent Install Log dated: $now" > $nbilog
echo "Networkbench Agent Install sysinfo dated: $now" > $nbsysinfo
echo "VERSION: ${nbversion}" >> $nbilog
echo "VERSION: ${nbversion}" >> $nbsysinfo
echo "INSTALL_LOCATION: ${prefix}" >> $nbilog
echo "ARCH: ${arch}" >> $nbsysinfo
nbuname=`uname -a 2>/dev/null`
echo "UNAME: $nbuname" >> $nbsysinfo
env | sort | sed -e 's/^/ENV: /' >> $nbsysinfo


error() {
    echo "ERROR: $@" >> $nbilog
    echo -e "\033[31mERROR: $@" ; tput sgr0 >&2
  cat >&2 <<EOF
  
安装失败：请将安装日志 $logtar 发送给听云技术支持

ERROR: If the log file $logtar exists please attach it to your bug report.

EOF
}

fatal() {
    echo "FATAL: $@" >> $nbilog
    echo -e "\033[31mFATAL: $@" ; tput sgr0 >&2
    cat >&2 <<EOF
  
安装失败：请将安装日志 $logtar 发送给听云技术支持

FATAL: If the log file $logtar exists please attach it to your bug report.

EOF
    exit 1
}
#写日志api
log() {
    echo "$@" >> $nbilog
}
#执行命令并将命令和结果输出到日志
logcmd() {
    echo "Executing: $@" >> $nbilog
    $@ >> $nbilog 2>&1
    ret=$?
    echo "Command returned: $ret" >> $nbilog
    return $ret
}
#添加路径到PATH环境变量
add_to_path() {
    case "${PATH}" in
    $1:*) ;;
    *:$1) ;;
    *:$1:*) ;;
    *) [ -d "$1" ] && PATH="${PATH}${PATH_SEPARATOR}$1" ;;
  esac
}
#添加搜索路径
add_to_path /usr/local/bin
add_to_path /usr/local/php
add_to_path /usr/local/php/bin
add_to_path /usr/local/zend/bin
add_to_path /usr/php/bin
add_to_path /opt/php/bin
add_to_path /opt/zend/bin
add_to_path /opt/local/bin

if [ -n "${php_path}" ]; then
    oIFS="${IFS}"
    IFS=$PATH_SEPARATOR
    set -- $php_path
    while [ -n "$1" ]; do
    add_to_path $1
    shift
    done
    IFS="${oIFS}"
fi

# 查找php实例
log "Final PATH=$PATH"
nbphplist=
oIFS="${IFS}"
IFS=${PATH_SEPARATOR}
set -- $PATH
while [ -n "$1" ]; do
    pe=$1; shift
    found=0
    [ -x "${pe}/php" ] && [ -f "${pe}/php" ] && {
      log "Found PHP in ${pe}"
      nbphplist="${nbphplist}${PATH_SEPARATOR}${pe}"
      nbphplist=${nbphplist#${PATH_SEPARATOR}}
      found=1
    }
    [ $found -eq 0 ] && {
      log "No PHP found in ${pe}"
    }
done
show_info="PHP install path not found, please enter the path:"
if [ -z "${nbphplist}" ]; then
  while [ -z "${nbphplist}" ]; do
    clear
    echo "==================================================="
    echo ${en} "${show_info} ${ec}"
    read pe
    found=0
    [ -x "${pe}/php" ] && [ -f "${pe}/php" ]; {
      log "Found PHP in ${pe}"
      nbphplist="${nbphplist}${PATH_SEPARATOR}${pe}"
      nbphplist=${nbphplist#${PATH_SEPARATOR}}
    }
    [ $found -eq 0 ] && {
      show_info="not found php in path [${pe}], please enter the PHP path:"
    }
  done
fi

IFS="${oIFS}"

# 显示banner
    bold=`tput bold 2> /dev/null`
    rmso=`tput sgr0 2> /dev/null`

banner() {
    clear
    echo "${bold}Networkbench PHP Agent Installation (interactive mode)${rmso}"
    echo "==================================================="
    echo ""
}


log "Final PHP list is: $nbphplist"
if [ -z "${nbphplist}" ]; then
    numphp=0
else
    oIFS="$IFS"
    IFS=${PATH_SEPARATOR}
    set -- $nbphplist
    numphp=$#
    IFS="${oIFS}"
fi

# 读取输入数字选项
choose_list() {
    prompt="$1"; shift
    upper=$1; shift
    valid=0
    chosen=
    while [ $valid -eq 0 ]; do
    echo ${en} "${prompt} (1-${upper}, 0 to exit): ${ec}"
    read choice
    case "${choice}" in
      0 | [1-9] | [1-9][0-9] | [1-9][0-9][0-9]) ;;
      *) choice= ;;
      esac
      if [ -n "${choice}" ]; then
      if [ $choice -ge 0 -a $choice -le $upper ]; then
        valid=1
      fi
      fi
  done
  chosen="${choice}"
  return $choice
}

# 命令行参数未指定安装还是卸载,要求用户确认
if [ -z "${imode}" ]; then
  banner
  cat <<EOF
Please select from one of the following options:

1)  Install (安装)
2)  Uninstall (卸载)

0)  Exit (取消)

EOF
  choose_list "   Enter choice" 2
  case ${chosen} in
    0)  exit 0 ;;
    2)  imode="uninstall" ;;
    *)  imode="install" ;;
  esac
fi

log "Install mode: ${imode}"

# 验证是否有可用php, 如果没有则提示

get_inst() {
  gin=1
  gioIFS="$IFS"
  IFS=${PATH_SEPARATOR}
  for giod in $nbphplist; do
    if [ $gin -eq $1 ]; then
      echo "${giod}"
      break
    fi
    gin=$(($gin+1))
  done
  IFS="${gioIFS}"
}

# 显示PHP列表
disp_get_php_list() {
  phpulist=
  if [ $numphp -eq 0 ]; then
    return 0
  fi
  
  if [ $numphp -eq 1 ]; then
    phpulist="${nbphplist}"
    return 0
  fi
  
  pln=0
  oIFS="$IFS"
  IFS=${PATH_SEPARATOR}
  spaces="   "
  for d in $nbphplist; do
    pln=$(($pln+1))
    IFS="$oIFS"
    [ $pln -eq 10 ] && spaces= "  "
    echo "   ${pln})${spaces}${d}"
  done
  IFS="$oIFS"
  echo ""
  echo "   0)   Exit"
  echo ""
  valid=0
  while [ $valid -eq 0 ]; do
    echo ${en} "   Selection (1-${pln}, 0 to exit or all): ${ec}"
    read sel
    oIFS="$IFS"
    IFS=", "
    if [ -n "$sel" ]; then
      valid=1
      for e in $sel; do
        case "$e" in
          0) 
            exit 0 ;;
          [Aa][Ll][Ll]) 
            phpulist="${nbphplist}" ;;
          [1-9] | [1-9][0-9] | [1-9][0-9][0-9])
            if [ $e -gt $pln ]; then
              valid=0
            else
              td=`get_inst $e`
              phpulist="${phpulist}:$td"
              phpulist=${phpulist#:}
            fi
            ;;
          *) 
            valid=0 ;;
        esac
      done
    else
      valid=0
    fi
    IFS="$oIFS"
  done
  echo ""
  return 0
}

get_fpm_ini() {
  local ret="${1%\/cli\/php.ini}"
  [ "${ret}/cli/php.ini" = "${1}" ] && [ -f "${ret}/fpm/php.ini" ] && echo "${ret}/fpm/php.ini"
}
inifile_by_options() {
  while [ -n "$1" ]; do
    case $1 in
      --with-config-file-path=*) echo "${1#*=}"
      ;;
      *);;
    esac
    shift
  done
}
inidir_by_options() {
  while [ -n "$1" ]; do
    case $1 in
      --with-config-file-scan-dir=*) echo "${1#*=}"
      ;;
      *);;
    esac
    shift
  done
}
print_php_info() {
  local s1=`echo "<?php print phpversion(); print ${2}; ?>"| ${1} 2>/dev/null`
  local s2=`echo '<?php print phpversion(); ?>'| ${1} 2>/dev/null`
  echo ${s1/$s2}
}
print_extension_dir() {
  local s1=`${1} -r "print phpversion(); print ini_get('extension_dir'); " 2>/dev/null`
  local s2=`${1} -r "print phpversion();" 2>/dev/null`
  echo ${s1/$s2}
}
gather_info() {
  pdir="$1"
  havebin=
  pi_bin=
  
  [ -x "${pdir}/php" ] && pi_bin="${pdir}/php"
  
  log "${pdir}: pi_bin=${pi_bin}"
  phpi=
  [ -n "${pi_bin}" ] && {
    havebin=1
    phpi=/tmp/nbinstall-$$.phpi
    rm -f ${phpi} > /dev/null 2>&1
    "${pi_bin}" -i > ${phpi} 2>&1
    log "${pdir}: php -i output follows {"
    cat ${phpi} >> ${nbilog} 2> /dev/null
    log "${pdir}: php -i output done }"
  }
  log "${pdir}: havebin=${havebin}"
  pi_ver=
  [ -n "${havebin}" ] && pi_ver=`print_php_info ${pi_bin} "phpversion()"`
  log "${pdir}: pi_ver=${pi_ver}"
  if [ -z "${pi_ver}" ]; then
    log "${pdir}: couldn't determine version"
    error "could not determine the version of PHP located at: ${pdir}${nl}没有发现${pdir}的PHP版本信息${nl}"
    return 1
  fi
  case "${pi_ver}" in
    5.3.* | 5.4.* | 5.5.* | 5.6.* | 7.0.* | 7.1.*) ;;
    *) error "unsupported version '${pi_ver}' of PHP found at: ${pdir}${nl}本探针不支持${pi_ver}版本的PHP${nl}"
      log "${pdir}: unsupported version '${pi_ver}'"
      return 1 
      ;;
  esac
  pi_extdir=
  pi_cli_inidir=
  pi_cli_inifile=
  [ -n "${havebin}" ] && pi_extdir=`print_extension_dir ${pi_bin}`
  [ -n "${havebin}" ] && pi_cli_inidir=`print_php_info ${pi_bin} "PHP_CONFIG_FILE_SCAN_DIR"`
  [ -n "${pi_cli_inidir}" -a ! -d "${pi_cli_inidir}" ] && pi_cli_inidir=
  [ -n "${havebin}" ] && pi_cli_inifile=`print_php_info ${pi_bin} "PHP_CONFIG_FILE_PATH"`
  [ -n "${pi_cli_inifile}" -a -d "${pi_cli_inifile}" ] && pi_cli_inifile="${pi_cli_inifile%/}/php.ini"
    
  log "${pdir}: pi_cli_inidir=${pi_cli_inidir}"
  log "${pdir}: pi_cli_inifile=${pi_cli_inifile}"
  pi_extdir="${pi_extdir%/}"
  if [ -z "${pi_extdir}" ]; then
    error "could not determine extension directory for the PHP located at:  ${pdir}"
    return 1
  fi
  if [ ! -d "${pi_extdir}" ]; then
    error "computed PHP extension directory: ${pi_extdir}${nl}which is for the PHP installation located at: ${pdir}  does not exist. ${nl}This particular instance of PHP will be skipped."
    return 1
  fi
  log "${pdir}: pi_extdir=${pi_extdir}"
  
  
  pi_so_inidir=
  pi_so_inifile=
  pi_extdir2=
  [ -x "${pdir}/php-config" ] && {
    pi_extdir2=`"${pdir}/php-config" --extension-dir`
    local options=`"${pdir}/php-config" --configure-options`
    pi_so_inifile=`inifile_by_options ${options}`
    pi_so_inidir=`inidir_by_options ${options}`
  }
  [ "${pi_extdir2}" = "${pi_extdir}" ] && pi_extdir2=
  [ -n "${pi_so_inidir}" -a ! -d "${pi_so_inidir}" ] && pi_so_inidir=
  [ -n "${pi_so_inifile}" -a -d "${pi_so_inifile}" ] && pi_so_inifile="${pi_so_inifile%/}/php.ini"
  log "${pdir}: pi_so_inidir=${pi_so_inidir}"
  log "${pdir}: pi_so_inifile=${pi_so_inifile}"
  log "${pdir}: pi_extdir2=${pi_extdir2}"
  
  
  
  pi_modver=
  case "${pi_ver}" in    
    5.3.*)  pi_modver="20090626" ;;
    5.4.*)  pi_modver="20100525" ;;
    5.5.*)  pi_modver="20121212" ;;
    5.6.*)  pi_modver="20131226" ;;
    7.0.*)  pi_modver="20151012" ;;
    7.1.*)  pi_modver="20160303" ;;
  esac
  log "${pdir}: pi_modver=${pi_modver}"
  #zts版本检测
  pi_zts=
  if [ -n "${havebin}" ]; then
    if grep 'Thread Safety' ${phpi} | grep 'enabled' > /dev/null 2>&1; then
    pi_zts=yes
    elif grep 'Thread Safety' ${phpi} | grep 'disabled' > /dev/null 2>&1; then
    pi_zts=no
    fi
  fi
  if [ -z "${pi_zts}" ]; then
    pi_zts="no"
    case "${pi_extdir}" in
      *non-zts* | *no-zts*) ;;
      *zts*)  pi_zts="yes" ;;
      *);;
    esac
  fi
  log "${pdir}: pi_zts=${pi_zts}"
  [ -n "${pi_cli_inifile}" -a -n "${pi_cli_inidir}" ] && pi_cli_inifile=
  [ -n "${pi_so_inifile}" -a -n "${pi_so_inidir}" ] && pi_so_inifile=
  #如果没有conf.d文件夹,则准备php.ini文件,将配置信息写入php.ini
  pi_fpm_inifile=
  [ -n "${pi_cli_inifile}" ] && pi_fpm_inifile=`get_fpm_ini ${pi_cli_inifile}`
  if [ -n "${pi_cli_inifile}" ]; then
    pi_inilist="${pi_inilist}${PATH_SEPARATOR}${pi_cli_inifile}"
    pi_inilist=${pi_inilist#${PATH_SEPARATOR}}
  fi
  if [ -n "${pi_so_inifile}" ]; then
    pi_inilist="${pi_inilist}${PATH_SEPARATOR}${pi_so_inifile}"
    pi_inilist=${pi_inilist#${PATH_SEPARATOR}}
  fi
  [ -n "${pi_fpm_inifile}" ] && {
    pi_inilist="${pi_inilist}${PATH_SEPARATOR}${pi_fpm_inifile}"
    pi_inilist=${pi_inilist#${PATH_SEPARATOR}}
  }
  log "${pdir}: final pi_cli_inifile=${pi_cli_inifile}"
  log "${pdir}: final pi_fpm_inifile=${pi_fpm_inifile}"
  log "${pdir}: final pi_cli_inidir=${pi_cli_inidir}"
  
  [ -n "${phpi}" ] && rm -f $phpi > /dev/null 2>&1
  
  return 0
}

fix_prefix() {
  repaire_prefix=${prefix//\//\\/}
  repaire_runtime_root=${runtime_root//\//\\/}
  sed -i "s/^[\t ]*nbs\.prefix.*$/nbs.prefix=\"${repaire_prefix}\"/" $1
  sed -i "s/^[\t ]*nbs\.runtime_root.*$/nbs.runtime_root=\"${repaire_runtime_root}\"/" $1
  sed -i "s/^;[\t ]*extension=networkbench.so/extension=networkbench.so/" $1
}

update_part() {
  local ini_dir=$1
  [ -n "${ini_dir}" -a ! -w "${ini_dir}" ] && {
      done_msg="${done_msg}\n${ini_dir} Write Permission deny."
      exit 1
  }

  if [ -n "${ini_dir}" ]; then
    if sed -e "s/REPLACE_WITH_REAL_KEY/${nb_license}/" "${ilibdir}/scripts/networkbench.ini.template" > "${ini_dir}/networkbench.ini"; then
      logcmd chmod 644 "${ini_dir}/networkbench.ini"
      sed -e "s/PHP Application/${nb_appname}/" "${ini_dir}/networkbench.ini" > "${ini_dir}/networkbench.ini"
      echo "      Install Status : ${ini_dir}/networkbench.ini created"
      fix_prefix "${ini_dir}/networkbench.ini"
    else
      log "${pdir}: copy ini template to ${ini_dir}/networkbench.ini failed"
      return 0
    fi
  fi
  if [ -n "${ini_dir}" -a -f "${ini_dir}/networkbench.ini" ]; then
    pi_nbinilist="${pi_nbinilist}${PATH_SEPARATOR}${ini_dir%/}/networkbench.ini"
    pi_nbinilist=${pi_nbinilist#${PATH_SEPARATOR}}
  fi
  update_count=$((update_count+1))
  return 1
}
get_fpm() {
  local ret="${1%\/cli\/conf.d}"
  [ "${ret}/cli/conf.d" = "${1}" ] && [ -f "${ret}/fpm/php.ini" ] && [ -d "${ret}/fpm/conf.d" ] && echo "${ret}/fpm/conf.d"
}
update_ini() {
  fpm_dir=`get_fpm ${pi_cli_inidir}`
  update_count=0
  update_part ${pi_cli_inidir}
  [ -n "${fpm_dir}" ] && [ "${fpm_dir}" != "${pi_cli_inidir}" ] && update_part ${fpm_dir}
  [ -n "${pi_so_inidir}" ] && [ "${pi_so_inidir}" != "${pi_cli_inidir}" ] && [ "${pi_so_inidir}" != "${fpm_dir}" ] && update_part ${pi_so_inidir}
  [ $update_count -lt 1 ] && {
    istat="failed"
    log "${pdir}: copy ini template to ${pi_cli_inidir}/networkbench.ini failed"
  }
}
install_agent_here() {
  zts=
  istat=
  [ "${pi_zts}" = "yes" ] && zts="-zts"
  srcf="${ilibdir}/agent/${arch}/networkbench-${pi_modver}${zts}.so"
  destf="${pi_extdir}/networkbench.so"


  logcmd rm -f "${destf}"
  logcmd ln -sf "${srcf}" "${destf}" || {
    istat="failed"
    log "${pdir}: link '${srcf}' -> '${destf}' FAILED!"
    done_msg="${done_msg}\nlink failed! please run this as root: ln -sf \"${srcf}\" \"${destf}\""
  }
  [ -n "${pi_extdir2}" ] && {
    destf="${pi_extdir2}/networkbench.so"
    logcmd rm -f "${destf}"
    logcmd ln -sf "${srcf}" "${destf}" || {
      istat="failed"
      log "${pdir}: link '${srcf}' -> '${destf}' FAILED!"
      done_msg="${done_msg}\nlink failed! please run this as root: ln -sf \"${srcf}\" \"${destf}\""
    }
  }
  update_ini
  
  if [ -z "${istat}" ]; then
    istat="OK"
    success="yes"
    echo "      Install Status : ${istat}"
cat <<EOF

安装成功, 请重启apache或php-fpm

${bold}Networkbench${rmso} is now installed on your system.
you may need to restart Apache or php-fpm. 

本次安装过程的日志在以下位置

The install log file location is:
$logtar

如果探针运行正常，会产生以下日志文件:

If everything is working properly you should see the following two log files:

${runtime_root}/log/networkbench/daemon.log
${runtime_root}/log/networkbench/php_agent.log

EOF
    
  else
    echo "      Install Status : ${istat}"
cat <<EOF

安装失败：请将安装日志 $logtar 发送给听云技术支持

Fail: The Networkbench agent is not installed.
      If the log file $logtar exists please attach it to your bug report.

EOF
  fi
  
  log "${pdir}: install status: ${istat}"
  
}

[ -z "${runtime_root}" ] && runtime_root="/var"

do_install() {

  banner

  while [ ! -w $runtime_root ]; do
	echo "${runtime_root} could not be written or does not exists."
    echo ${en} "Enter Runtime Files Path(可写权限的运行时路径，用于存放日志和pid文件):${ec}"
    read runtime_root    
  done
  [ ! -d "${runtime_root}/log" ] && logcmd mkdir "${runtime_root}/log"
  [ ! -d "${runtime_root}/log/networkbench" ] && logcmd mkdir "${runtime_root}/log/networkbench"
  logcmd chmod 0777 "${runtime_root}/log/networkbench"
  
  [ -z "${nb_license}" ] && {
    echo ${en} "Enter license key (请录入授权码): ${ec}"
    read nb_license
  }
  
  [ -z "${nb_license}" ] && nb_license="REPLACE_WITH_REAL_KEY"
  [ -z "${nb_appname}" ] && {      
    echo ${en} "Enter the name of website (请录入网站名称): ${ec}"
    read nb_appname
  }
  [ -z "${nb_appname}" ] && nb_appname="PHP Application"
  
  if [ $numphp -gt 1 ]; then
    cat <<EOF
      
计算机上存在多个版本的PHP,请选择您要安装探针对应的PHP位置, 
如果选择多个选项, 请使用空格分隔, 或录入all全部安装
Below is a list of the directories in which we found a copy of PHP. Please
select the directory or directories for which you wish to install ${bold}Networkbench${rmso}.
You can select either a single directory or multiple directories by separating
each choice with either a space or a comma. To select all of the directories
shown, please enter the special keyword 'all' (without the quotes).

EOF
  fi
  
  disp_get_php_list
  pi_inilist=
  pi_nbinilist=
  
  if [ -n "${phpulist}" ]; then
    ioIFS="$IFS"
    IFS="$PATH_SEPARATOR"
    for pdir in $phpulist; do
      IFS="${ioIFS}"
      echo "====== install agent for ${pdir} ======"
      log "install agent for ${pdir} "
      gather_info "${pdir}" || continue

      echo ""
      echo "Found a valid PHP in : ${pdir}"
      echo "         PHP Version : ${pi_ver}"
      echo "  Module API version : ${pi_modver}"
      echo "    Module directory : ${pi_extdir}"
      echo "  Zend Thread Safety : ${pi_zts}"
      [ -n "${pi_cli_inidir}" ] && echo "   CLI ini directory : ${pi_cli_inidir}"
      [ -n "${pi_cli_inifile}" ] && echo "   CLI ini file : ${pi_cli_inifile}"
  
      install_agent_here      
    done
  IFS="${ioIFS}"
  fi
  
  pi_upglist="${pi_inilist}${PATH_SEPARATOR}${pi_nbinilist}"
  pi_upglist=${pi_upglist#${PATH_SEPARATOR}}
  pi_upglist=${pi_upglist%${PATH_SEPARATOR}}
  
  if [ -n "${pi_upglist}" ]; then
    ioIFS="${IFS}"
    IFS=${PATH_SEPARATOR}
    for ifile in ${pi_upglist}; do
      if [ -f "${ifile}" ]; then
      
        if grep -q "^[\t ]*nbs.license_key" "${ifile}" > /dev/null 2>&1; then
            if [ $nb_license != "REPLACE_WITH_REAL_KEY" ]; then
                 sed -i "s/^[\t ]*nbs.license_key.*$/nbs.license_key=\"${nb_license}\"/" "${ifile}"
            fi
        else
            sed -e "s/REPLACE_WITH_REAL_KEY/${nb_license}/" "${ilibdir}/scripts/networkbench.ini.template" >> "${ifile}"
        fi
        
         if grep -q "^[\t ]*nbs.app_name" "${ifile}" > /dev/null 2>&1; then
            if [ $nb_appname != "PHP Application" ]; then
                 sed -i "s/^[\t ]*nbs.app_name.*$/nbs.app_name=\"${nb_appname}\"/" "${ifile}"
            fi
        else
            sed -e "s/PHP Application/${nb_appname}/" "${ilibdir}/scripts/networkbench.ini.template" >> "${ifile}"
        fi

        fix_prefix "${ifile}"

      fi
    done
    IFS="${ioIFS}"
  fi
    
  return 0
}



remove_from_here() {  
  destf="${pi_extdir}/networkbench.so"
  [ -f "${destf}" ] && logcmd rm -f "${destf}"
  [ -n "${pi_extdir2}" ] && {
    destf="${pi_extdir2}/networkbench.so"
    [ -f "${destf}" ] && logcmd rm -f "${destf}"
  }
  ini_path_name=`dirname ${pi_cli_inifile}`
  [ -n "${ini_path_name}" -a -f "${ini_path_name}/networkbench.ini" ] && logcmd rm -f "${ini_path_name}/networkbench.ini"
  [ -n "${ini_path_name}" -a -f "${ini_path_name}/php.ini" ] && sed -i "s/^extension=networkbench.so$/;extension=networkbench.so/" "${ini_path_name}/php.ini"
  [ -n "${pi_cli_inidir}" -a -f "${pi_cli_inidir}/networkbench.ini" ] && logcmd rm -f "${pi_cli_inidir}/networkbench.ini"
  [ -n "${pi_cli_inidir}" -a -f "${pi_cli_inidir}/php.ini" ] && sed -i "s/^.*extension=networkbench.so$/;extension=networkbench.so/" "${pi_cli_inidir}/php.ini"
  [ -n "${pi_so_inidir}" -a -f "${pi_so_inidir}/networkbench.ini" ] && logcmd rm -f "${pi_so_inidir}/networkbench.ini"
  [ -n "${pi_so_inidir}" -a -f "${pi_so_inidir}/php.ini" ] && sed -i "s/^.*extension=networkbench.so$/;extension=networkbench.so/" "${pi_so_inidir}/php.ini"
  pi_fpm_inidir=`get_fpm ${pi_cli_inidir}`
  [ -n "${pi_fpm_inidir}" -a -f "${pi_fpm_inidir}/networkbench.ini" ] && logcmd rm -f "${pi_fpm_inidir}/networkbench.ini"
  [ -n "${pi_fpm_inidir}" -a -f "${pi_fpm_inidir}/php.ini" ] && sed -i "s/^.*extension=networkbench.so$/; extension=networkbench.so/" "${pi_fpm_inidir}/php.ini"

}

do_uninstall() {
  banner
  phpulist="${nbphplist}"
  pi_inilist=
  if [ -n "${phpulist}" ]; then
    ioIFS="$IFS"
    IFS=${PATH_SEPARATOR}
    for pdir in $phpulist; do
      IFS="${ioIFS}"
      gather_info "${pdir}" || continue
      echo "Removing from this PHP : ${pdir}"
      echo "      Module directory : ${pi_extdir}"

      remove_from_here
      
      echo ""
    done
    IFS="${ioIFS}"
  fi  
}

[ "${imode}" = "install" ] && do_install
[ "${imode}" = "uninstall" ] && do_uninstall

exit 0
